'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var isString = _interopDefault(require('lodash/isString'));
var isArray = _interopDefault(require('lodash/isArray'));
var isObject = _interopDefault(require('lodash/isObject'));
var isNumber = _interopDefault(require('lodash/isNumber'));
var Utils = require('conflux-web-utils');

var outputBigNumberFormatter = function outputBigNumberFormatter(number) {
  return Utils.toBN(number).toString(10);
};
var isPredefinedEpochNumber = function isPredefinedEpochNumber(epochNumber) {
  return epochNumber === 'latest_state' || epochNumber === 'latest_mined' || epochNumber === 'earliest';
};
var inputDefaultEpochNumberFormatter = function inputDefaultEpochNumberFormatter(epochNumber, moduleInstance) {
  if (epochNumber === undefined || epochNumber === null) {
    return moduleInstance.defaultEpoch;
  }
  return inputBlockAddressFormatter(epochNumber);
};
var inputEpochNumberFormatter = function inputEpochNumberFormatter(epochNumber) {
  if (epochNumber === undefined || epochNumber === null || isPredefinedEpochNumber(epochNumber)) {
    return epochNumber;
  }
  return Utils.numberToHex(epochNumber);
};
var inputBlockAddressFormatter = function inputBlockAddressFormatter(blockAddress) {
  if (blockAddress === undefined || blockAddress === null || isPredefinedEpochNumber(blockAddress)) {
    return blockAddress;
  }
  if (Utils.isHexStrict(blockAddress)) {
    if (isString(blockAddress)) {
      return blockAddress.toLowerCase();
    }
    return blockAddress;
  }
  return Utils.numberToHex(blockAddress);
};
var txInputFormatter = function txInputFormatter(txObject) {
  if (txObject.to) {
    txObject.to = inputAddressFormatter(txObject.to);
  }
  if (txObject.data && txObject.input) {
    throw new Error('You can\'t have "data" and "input" as properties of transactions at the same time, please use either "data" or "input" instead.');
  }
  if (!txObject.data && txObject.input) {
    txObject.data = txObject.input;
    delete txObject.input;
  }
  if (txObject.data && !Utils.isHex(txObject.data)) {
    throw new Error('The data field must be HEX encoded data.');
  }
  if (txObject.gas || txObject.gasLimit) {
    txObject.gas = txObject.gas || txObject.gasLimit;
  }
  ['gasPrice', 'gas', 'value', 'nonce'].filter(function (key) {
    return txObject[key] !== undefined;
  }).forEach(function (key) {
    txObject[key] = Utils.numberToHex(txObject[key]);
  });
  return txObject;
};
var inputCallFormatter = function inputCallFormatter(txObject, moduleInstance) {
  txObject = txInputFormatter(txObject);
  var from = moduleInstance.defaultAccount;
  if (txObject.from) {
    from = txObject.from;
  }
  if (from) {
    txObject.from = inputAddressFormatter(from);
  }
  if (txObject.hash === null || txObject.hash === undefined) {
    txObject.hash = '0x3d5cb19882200c8e1801a088bc9603abdd549212fd0e0a904da4fa7fa4d8c5e3';
  }
  if (txObject.blockHash === undefined) {
    txObject.blockHash = null;
  }
  if (txObject.transactionIndex === undefined) {
    txObject.transactionIndex = null;
  }
  if (txObject.contractCreated === null || txObject.contractCreated === undefined) {
    txObject.contractCreated = null;
  }
  if (txObject.r === null || txObject.r === undefined) {
    txObject.r = '0x0';
  }
  if (txObject.s === null || txObject.s === undefined) {
    txObject.s = '0x0';
  }
  if (txObject.v === null || txObject.v === undefined) {
    txObject.v = '0x0';
  }
  if (txObject.nonce === null || txObject.nonce === undefined) {
    txObject.nonce = '0x0';
  }
  if (txObject.value === null || txObject.value === undefined) {
    txObject.value = '0x0';
  }
  if (txObject.gas === null || txObject.gas === undefined) {
    txObject.gas = '0x1000000';
  }
  return txObject;
};
var inputTransactionFormatter = function inputTransactionFormatter(txObject, moduleInstance) {
  txObject = txInputFormatter(txObject);
  if (!isNumber(txObject.from) && !isObject(txObject.from)) {
    if (!txObject.from) {
      txObject.from = moduleInstance.defaultAccount;
    }
    if (!txObject.from && !isNumber(txObject.from)) {
      throw new Error('The send transactions "from" field must be defined!');
    }
    txObject.from = inputAddressFormatter(txObject.from);
  }
  return txObject;
};
var inputSignFormatter = function inputSignFormatter(data) {
  return Utils.isHexStrict(data) ? data : Utils.utf8ToHex(data);
};
var outputTransactionFormatter = function outputTransactionFormatter(receipt) {
  if (receipt.transactionIndex !== null) {
    receipt.transactionIndex = Utils.hexToNumber(receipt.transactionIndex);
  }
  if (receipt.gasPrice) {
    receipt.gasPrice = outputBigNumberFormatter(receipt.gasPrice);
  }
  if (receipt.value) {
    receipt.value = outputBigNumberFormatter(receipt.value);
  }
  receipt.nonce = Utils.hexToNumber(receipt.nonce);
  receipt.gas = Utils.hexToNumber(receipt.gas);
  if (receipt.to && Utils.isAddress(receipt.to)) {
    receipt.to = Utils.toChecksumAddress(receipt.to);
  } else {
    receipt.to = null;
  }
  if (receipt.from) {
    receipt.from = Utils.toChecksumAddress(receipt.from);
  }
  receipt.v = Utils.hexToNumber(receipt.v);
  return receipt;
};
var outputTransactionReceiptFormatter = function outputTransactionReceiptFormatter(receipt) {
  if (receipt.epochNumber !== null) {
    receipt.epochNumber = Utils.hexToNumber(receipt.epochNumber);
  }
  if (receipt.transactionIndex !== null) {
    receipt.transactionIndex = Utils.hexToNumber(receipt.transactionIndex);
  }
  receipt.cumulativeGasUsed = Utils.hexToNumber(receipt.cumulativeGasUsed);
  receipt.gasUsed = Utils.hexToNumber(receipt.gasUsed);
  if (isArray(receipt.logs)) {
    receipt.logs = receipt.logs.map(outputLogFormatter);
  }
  if (receipt.contractAddress) {
    receipt.contractAddress = Utils.toChecksumAddress(receipt.contractAddress);
  }
  if (typeof receipt.status !== 'undefined' && receipt.status !== null) {
    receipt.status = Boolean(parseInt(receipt.status));
  } else {
    receipt.status = true;
  }
  return receipt;
};
var outputBlockFormatter = function outputBlockFormatter(block) {
  block.gasLimit = Utils.hexToNumber(block.gasLimit);
  block.size = Utils.hexToNumber(block.size);
  block.timestamp = Utils.hexToNumber(block.timestamp);
  if (block.height !== null) {
    block.height = Utils.hexToNumber(block.height);
  }
  if (block.epochNumber !== null) block.epochNumber = Utils.hexToNumber(block.epochNumber);
  if (block.difficulty) {
    block.difficulty = outputBigNumberFormatter(block.difficulty);
  }
  if (isArray(block.transactions)) {
    block.transactions.forEach(function (item) {
      if (!isString(item)) return outputTransactionFormatter(item);
    });
  }
  if (block.miner) {
    block.miner = Utils.toChecksumAddress(block.miner);
  }
  return block;
};
var inputLogFormatter = function inputLogFormatter(options) {
  var toTopic = function toTopic(value) {
    if (value === null || typeof value === 'undefined') {
      return null;
    }
    value = String(value);
    if (value.indexOf('0x') === 0) {
      return value;
    }
    return Utils.fromUtf8(value);
  };
  if (options.fromBlock) {
    options.fromBlock = inputBlockAddressFormatter(options.fromBlock);
  }
  if (options.toBlock) {
    options.toBlock = inputBlockAddressFormatter(options.toBlock);
  }
  options.topics = options.topics || [];
  options.topics = options.topics.map(function (topic) {
    return isArray(topic) ? topic.map(toTopic) : toTopic(topic);
  });
  toTopic = null;
  if (options.address) {
    if (isArray(options.address)) {
      options.address = options.address.map(function (addr) {
        return inputAddressFormatter(addr);
      });
    } else {
      options.address = inputAddressFormatter(options.address);
    }
  }
  return options;
};
var outputLogFormatter = function outputLogFormatter(log) {
  if (typeof log.blockHash === 'string' && typeof log.transactionHash === 'string' && typeof log.logIndex === 'string') {
    var shaId = Utils.sha3(log.blockHash.replace('0x', '') + log.transactionHash.replace('0x', '') + log.logIndex.replace('0x', ''));
    shaId.replace('0x', '').substr(0, 8);
    log.id = "log_".concat(shaId);
  } else if (!log.id) {
    log.id = null;
  }
  if (log.epochNumber !== null) {
    log.epochNumber = Utils.hexToNumber(log.epochNumber);
  }
  if (log.transactionIndex !== null) {
    log.transactionIndex = Utils.hexToNumber(log.transactionIndex);
  }
  if (log.logIndex !== null) {
    log.logIndex = Utils.hexToNumber(log.logIndex);
  }
  if (log.address) {
    log.address = Utils.toChecksumAddress(log.address);
  }
  return log;
};
var inputPostFormatter = function inputPostFormatter(post) {
  if (post.ttl) {
    post.ttl = Utils.numberToHex(post.ttl);
  }
  if (post.workToProve) {
    post.workToProve = Utils.numberToHex(post.workToProve);
  }
  if (post.priority) {
    post.priority = Utils.numberToHex(post.priority);
  }
  if (!isArray(post.topics)) {
    post.topics = post.topics ? [post.topics] : [];
  }
  post.topics = post.topics.map(function (topic) {
    return topic.indexOf('0x') === 0 ? topic : Utils.fromUtf8(topic);
  });
  return post;
};
var outputPostFormatter = function outputPostFormatter(post) {
  post.expiry = Utils.hexToNumber(post.expiry);
  post.sent = Utils.hexToNumber(post.sent);
  post.ttl = Utils.hexToNumber(post.ttl);
  post.workProved = Utils.hexToNumber(post.workProved);
  if (!post.topics) {
    post.topics = [];
  }
  post.topics = post.topics.map(function (topic) {
    return Utils.toUtf8(topic);
  });
  return post;
};
var inputAddressFormatter = function inputAddressFormatter(address) {
  if (Utils.isAddress(address)) {
    return "0x".concat(address.toLowerCase().replace('0x', ''));
  }
  throw new Error("Provided address \"".concat(address, "\" is invalid, the capitalization checksum test failed."));
};
var outputSyncingFormatter = function outputSyncingFormatter(result) {
  result.startingBlock = Utils.hexToNumber(result.startingBlock);
  result.currentBlock = Utils.hexToNumber(result.currentBlock);
  result.highestBlock = Utils.hexToNumber(result.highestBlock);
  if (result.knownStates) {
    result.knownStates = Utils.hexToNumber(result.knownStates);
    result.pulledStates = Utils.hexToNumber(result.pulledStates);
  }
  return result;
};

var Formatters = /*#__PURE__*/Object.freeze({
    outputBigNumberFormatter: outputBigNumberFormatter,
    isPredefinedEpochNumber: isPredefinedEpochNumber,
    inputDefaultEpochNumberFormatter: inputDefaultEpochNumberFormatter,
    inputEpochNumberFormatter: inputEpochNumberFormatter,
    inputBlockAddressFormatter: inputBlockAddressFormatter,
    txInputFormatter: txInputFormatter,
    inputCallFormatter: inputCallFormatter,
    inputTransactionFormatter: inputTransactionFormatter,
    inputSignFormatter: inputSignFormatter,
    outputTransactionFormatter: outputTransactionFormatter,
    outputTransactionReceiptFormatter: outputTransactionReceiptFormatter,
    outputBlockFormatter: outputBlockFormatter,
    inputLogFormatter: inputLogFormatter,
    outputLogFormatter: outputLogFormatter,
    inputPostFormatter: inputPostFormatter,
    outputPostFormatter: outputPostFormatter,
    inputAddressFormatter: inputAddressFormatter,
    outputSyncingFormatter: outputSyncingFormatter
});

exports.formatters = Formatters;
